IF OBJECT_ID('TR__Environments__CircularParentCheck') IS NOT NULL
DROP TRIGGER [TR__Environments__CircularParentCheck]
GO

CREATE TRIGGER [TR__Environments__CircularParentCheck]
ON [Environments] AFTER UPDATE, INSERT AS
BEGIN

	SET NOCOUNT ON

	DECLARE @Environments TABLE(
		[Base_Environment_Id] INT NOT NULL, 

		[Resolution_Depth] INT NOT NULL,
		[Environment_Id] INT NOT NULL, 
		[Parent_Environment_Id] INT NULL
	)

	DECLARE @depth INT, @row_count INT
	    SET @depth = 1
	    SET @row_count = 1 
	 
	INSERT INTO @Environments 
	  ([Base_Environment_Id], [Resolution_Depth], [Environment_Id], [Parent_Environment_Id])
	SELECT 
	   [Environment_Id], 1, [Environment_Id], [Parent_Environment_Id]
	  FROM [inserted] 
     

	WHILE @row_count > 0
	BEGIN 
		SET @depth = @depth + 1

		-- insert all of the previous depth's parents from the actual table
		INSERT INTO @Environments 
		  ([Base_Environment_Id], [Resolution_Depth], [Environment_Id], [Parent_Environment_Id])
		SELECT
		  [Base_Environment_Id], @depth, AG.[Parent_Environment_Id], PAG.[Parent_Environment_Id]
		FROM @Environments AG
	         INNER JOIN [Environments] PAG
		             ON PAG.[Environment_Id] = AG.[Parent_Environment_Id]
		WHERE AG.[Resolution_Depth] = @depth - 1

		SET @row_count = @@ROWCOUNT
	
		-- if an environment is an ancestor of itself then shed tears
		IF EXISTS( SELECT * 
		             FROM [inserted] I
		                  INNER JOIN @Environments AG
		                          ON AG.[Resolution_Depth] > 1
		                         AND I.[Environment_Id] = AG.[Base_Environment_Id]
		                         AND I.[Environment_Id] = AG.[Parent_Environment_Id] )

		BEGIN 
				RAISERROR('The specified parent environment ID will cause a cycle at depth %i, which is not supported.',16,2, @depth)
				ROLLBACK
				RETURN 
		END 

		-- sanity check
		IF @depth > 1000 BEGIN
				RAISERROR('Depth exceeded 1000.',16,1) 
				ROLLBACK
				RETURN 
		END 
	END 


END
GO