EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Applications_CreateOrUpdateDeployable',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Deployable_Id',
    /* Description_Text        */ 'Creates a deployable for a specified application, or updates an existing one'
GO


IF OBJECT_ID('[Applications_CreateOrUpdateDeployable]') IS NOT NULL DROP PROCEDURE [Applications_CreateOrUpdateDeployable]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Applications_CreateOrUpdateDeployable]
(
	@Application_Id INT,
	@Deployable_Name VARCHAR(50),
	@CreatedBy_User_Name USERNAME = NULL,
	@CreatedOn_Date DATETIME = NULL,
	@ModifiedBy_User_Name USERNAME = NULL,
	@ModifiedOn_Date DATETIME = NULL,
	@Deployable_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Deployable_Id IS NOT NULL
	BEGIN

		DECLARE @Old_Deployable_Name VARCHAR(50)
		SET @Old_Deployable_Name = (SELECT [Deployable_Name]
		                              FROM [Deployables]
									 WHERE [Deployable_Id] = @Deployable_Id)

		UPDATE [Deployables]
		   SET [Deployable_Name] = @Deployable_Name,
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE()
		 WHERE [Deployable_Id] = @Deployable_Id

		IF @@ROWCOUNT=0 RAISERROR('Deployable %i does not exist.',16,1,@Deployable_Id)

		IF @Old_Deployable_Name <> @Deployable_Name
		BEGIN

			-- Update deployable names in deployment plans
			UPDATE DPAG
			   SET DPAG.[Deployable_Name] = @Deployable_Name
			  FROM [DeploymentPlanActionGroups] DPAG
			       LEFT JOIN [ApplicationDeploymentPlans] ADP
				          ON ADP.[DeploymentPlan_Id] = DPAG.[DeploymentPlan_Id]
			 WHERE (DPAG.[Deployable_Application_Id] = @Application_Id AND DPAG.[Deployable_Name] = @Old_Deployable_Name)
			    OR (DPAG.[Deployable_Application_Id] IS NULL AND ADP.[Application_Id] = @Application_Id AND DPAG.[Deployable_Name] = @Old_Deployable_Name)

		END

		/* Raise Event */
		EXEC [Events_RaiseEvent] 'APPMDD',@Application_Id,@Deployable_Id

	END
	ELSE
	BEGIN
		INSERT INTO [Deployables]
		(
			[Application_Id],
			[Deployable_Name],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date]
		)
		VALUES
        (
			@Application_Id,
			@Deployable_Name,
			COALESCE(@CreatedBy_User_Name, dbo.Session_GetUserName()),
			COALESCE(@CreatedOn_Date, GETUTCDATE()),
			COALESCE(@ModifiedBy_User_Name, dbo.Session_GetUserName()),
			COALESCE(@ModifiedOn_Date, GETUTCDATE())
		)
		SET @Deployable_Id = SCOPE_IDENTITY()

		/* Raise Event */
		EXEC [Events_RaiseEvent] 'APPNWD',@Application_Id,@Deployable_Id
	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Applications_CreateOrUpdateDeployable] TO [BuildMasterUser_Role]
