EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Releases_CreateOrUpdateRelease',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a release or updates an existing one.'
GO

IF OBJECT_ID('[Releases_CreateOrUpdateRelease]') IS NOT NULL DROP PROCEDURE [Releases_CreateOrUpdateRelease]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Releases_CreateOrUpdateRelease]
(
	@Application_Id INT,
	@Release_Number RELEASENUMBER,
	@Pipeline_Name VARCHAR(100),
	@Target_Date DATETIME,
	@Release_Name VARCHAR(50),
    @ReleaseTemplate_Name VARCHAR(50) = NULL
) AS
BEGIN

IF REPLACE(@Release_Name,' ','') = ''
    SET @Release_Name = NULLIF(REPLACE(@Release_Name,' ',''),'')

BEGIN TRY
	BEGIN TRANSACTION

    DECLARE @Release_Id INT
    SELECT @Release_Id = [Release_Id] FROM [Releases] WHERE [Application_Id] = @Application_Id AND [Release_Number] = @Release_Number

	IF @Release_Id IS NULL
	BEGIN

		DECLARE @Release_Sequence INT
			SET @Release_Sequence = COALESCE((SELECT MAX([Release_Sequence])
			                                    FROM [Releases]
									           WHERE [Application_Id] = @Application_Id), 0) + 1

		INSERT INTO [Releases]
		(
			[Application_Id],
			[Release_Number],
			[Pipeline_Name],
			[ReleaseStatus_Name],
			[Release_Name],
			[CreatedBy_User_Name],
			[CreatedOn_Date],
			[ModifiedBy_User_Name],
			[ModifiedOn_Date],
            [Release_Sequence],
            [ReleaseTemplate_Name]
		)
		VALUES
		(
			@Application_Id,
			@Release_Number,
			@Pipeline_Name,
			'Active',
			NULLIF(LTRIM(RTRIM(@Release_Name)), ''),
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			dbo.Session_GetUserName(),
			GETUTCDATE(),
			@Release_Sequence,
            @ReleaseTemplate_Name
		)

        SET @Release_Id = SCOPE_IDENTITY()

        INSERT INTO [ReleaseTargetDates] ([Release_Id], [Target_Name], [Start_Date], [End_Date])
             SELECT @Release_Id, 'Target', @Target_Date, @Target_Date
              WHERE @Target_Date IS NOT NULL

		INSERT INTO [ReleaseConfigurationFiles]
		(
			[Release_Id],
			[ConfigurationFile_Id],
			[Version_Number]
		)
		SELECT @Release_Id,
			   CFV.[ConfigurationFile_Id],
			   MAX(CFV.[Version_Number])
		  FROM [ConfigurationFileVersions] CFV
		       INNER JOIN [ConfigurationFiles] CF
			           ON CF.[ConfigurationFile_Id] = CFV.[ConfigurationFile_Id]
			   INNER JOIN [Deployables] D
			           ON D.[Deployable_Id] = CF.[Deployable_Id]
		 WHERE D.[Application_Id] = @Application_Id
		 GROUP BY CFV.[ConfigurationFile_Id]

		EXEC [Events_RaiseEvent] 'RELNEW', @Application_Id, @Release_Number

	END
	ELSE
	BEGIN

		UPDATE [Releases]
		   SET [Release_Name] = NULLIF(LTRIM(RTRIM(@Release_Name)), ''),
			   [Pipeline_Name] = COALESCE(@Pipeline_Name, [Pipeline_Name]),
			   [ModifiedBy_User_Name] = dbo.Session_GetUserName(),
			   [ModifiedOn_Date] = GETUTCDATE(),
               [ReleaseTemplate_Name] = @ReleaseTemplate_Name
		 WHERE [Release_Id] = @Release_Id

		EXEC [Events_RaiseEvent] 'RELMOD', @Application_Id, @Release_Number

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Releases_CreateOrUpdateRelease] TO [BuildMasterUser_Role]

